<?php

function _neo4j_rule_context() {
  $context = _neo4j_get_context();
  $context['visit'] = _neo4j_page_visit();

  return $context;
}

function neo4j_action_run_cypher($query, $args, RulesState $state) {
  $client = neo4j_get_client();
  $context = array_merge(_neo4j_rule_context(), $state->variables['__query_params']);

  if ($client) {
    try {
      $client->run($query, $context);
    } catch (Exception $ex) {
      watchdog_exception('neo4j_rule', $ex);
    }
  }
}

class CypherQueryEvaluator extends RulesTokenEvaluator {

  public function prepare($text, $var_info) {
    $text = is_array($text) ? implode('', $text) : $text;
    // Skip this evaluator if there are no tokens.
    $this->setting = static::scan($text) ? TRUE : NULL;
  }

  public static function scan($text) {
    preg_match_all('/
    \{             # [ - pattern start
    ([a-zA-Z0-9-]*)   # match $type not containing whitespace __ [ or ]
    __             # __ - separator
    ([^\[\}]*)     # match $name not containing [ or ]
    \}             # ] - pattern end
    /x', $text, $matches);

    $types = $matches[1];
    $tokens = $matches[2];

    $results = [];
    for ($i = 0; $i < count($tokens); $i++) {
      $results[$types[$i]][$tokens[$i]] = $matches[0][$i];
    }

    return $results;
  }

  public function evaluate($text, $options, RulesState $state) {
    $var_info = $state->varInfo();
    $options += ['sanitize' => FALSE];

    $replacements = [];
    $data = [];
    // We also support replacing tokens in a list of textual values.
    $whole_text = is_array($text) ? implode('', $text) : $text;
    foreach (static::scan($whole_text) as $var_name => $tokens) {
      $var_name = str_replace('-', '_', $var_name);
      if (isset($var_info[$var_name]) && ($token_type = _rules_system_token_map_type($var_info[$var_name]['type']))) {
        // We have to key $data with the type token uses for the variable.
        $data = rules_unwrap_data([$token_type => $state->get($var_name)], [$token_type => $var_info[$var_name]]);
        $replacements += token_generate($token_type, $tokens, $data, $options);
      }
      else {
        $replacements += token_generate($var_name, $tokens, [], $options);
      }
      // Remove tokens if no replacement value is found. As token_replace() does
      // if 'clear' is set.
      $replacements += array_fill_keys($tokens, '');
    }

    // Optionally clean the list of replacement values.
    if (!empty($options['callback']) && function_exists($options['callback'])) {
      $function = $options['callback'];
      $function($replacements, $data, $options);
    }

    // Actually apply the replacements.
    $tokens = array_keys($replacements);
    $values = array_values($replacements);
    $context = array_combine(array_map(function($item) {
      return trim($item, '{}');
    }, $tokens), $values);
    $state->variables['__query_params'] = $context;

    return $text;
  }

  public static function help($var_info) {
    $help = parent::help($var_info);
    foreach ($help as $name => &$value) {
      if (strpos($name, '#') === 0) {
        continue;
      }

      if (!empty($value['#rows'])) {
        $orig_rows = $value['#rows'];
        $value['#rows'] = [];
        foreach ($orig_rows as $row) {
          $row[0] = static::translate($row[0]);

          $value['#rows'][$row[0]] = $row;
        }
      }
    }

    $help['#description'] = static::translate($help['#description']);

    $help['neo4j'] = [
      '#theme' => 'table',
      '#header' => [t('Token'), t('Label'), t('Description')],
      '#prefix' => '<h3>' . t('Replacement patterns for %label', ['%label' => t('Neo4j')]) . '</h3>',
      '#rows' => [
        '{visit}' => ['{visit}', t('Visit id'), t('The id of the current visit')],
        '{method}' => ['{method}', t('Method'), t('HTTP request method')],
        '{requestUri}' => ['{requestUri}', t('Request URI'), t('')],
        '{ip}' => ['{ip}', t('IP address'), t('IP address of the client')],
        '{referer}' => ['{referer}', t('HTTP referer'), t('')],
        '{uid}' => ['{uid}', t('Global user id'), t('')],
        '{roles}' => ['{roles}', t('User roles'), t('Roles of the currently logged in user')],
      ],
    ];

    return $help;
  }

  public static function translate($text) {
    return strtr($text, [
      '[' => '{',
      ']' => '}',
      ':' => '__',
    ]);
  }

}
